////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

// Create a new stylesheet, or load a .css file
HTMLDocument.prototype.createStyleSheet = function(xmlURI)
{
  // load the xml
  var theHeadNode = document.getElementsByTagName("head")[0];
  var theStyleNode = document.createElement('style');

  if (xmlURI) {
    var xmlHttp = getHTTPRequest();
    xmlHttp.open("GET", xmlURI, false);
    xmlHttp.send(null);

    var theTextNode = document.createTextNode(xmlHttp.responseText);

    theStyleNode.appendChild(theTextNode);
  }
  theHeadNode.appendChild(theStyleNode);
  return theStyleNode;
};

// Add a css rule
HTMLStyleElement.prototype.addRule = function(itemName, itemStyle)
{
  var theStyleNode = document.getElementsByTagName("style")[0];
  var theTextNode = document.createTextNode(itemName + " { " + itemStyle + " }");
  theStyleNode.appendChild(theTextNode);
};

/**
 * Add loading support to XML documents.
 *
 * TODO: return something
 */
XMLDocument.prototype.load = function(xmlURI)
{
  var childNodes = this.childNodes;
  for (var i = childNodes.length - 1; i >= 0; i--)
    this.removeChild(childNodes[i]);

  var newDOM = xopus_globs.xmllib.load(xmlURI);
  var newElt = this.importNode(newDOM.documentElement, true);
  this.appendChild(newElt);
};

XMLDocument.prototype.loadXML = function(xmlString)
{
  var childNodes = this.childNodes;
  for (var i = childNodes.length - 1; i >= 0; i--)
    this.removeChild(childNodes[i]);

  var dp = new DOMParser();
  var newDOM = dp.parseFromString(xmlString, "text/xml");
  var newElt = this.importNode(newDOM.documentElement, true);
  this.appendChild(newElt);
};

/**
 * Create a node with the specified type, name and namespaceURI
 */
XMLDocument.prototype.createNode = function(type, name, namespaceURI)
{
  switch(type)
  {
    // Element
    case 1:
      return this.createElementNS(namespaceURI, name);
    // Attribute
    case 2:
      return this.createAttributeNS(namespaceURI, name);

      //var attr = this.createAttributeNS(namespaceURI, name.substr(name.indexOf(":") + 1));
      //attr.prefix = name.substr(0, name.indexOf(":"));
      //__xopus_alert("prefix: " + name.substr(0, name.indexOf(":")) + " name: " + name.substr(name.indexOf(":") + 1));
      //return attr;
    // DocumentFragment
    case 11:
      return this.createDocumentFragment();
    default:
      __xopus_alert("Not implemented: XMLDocument.prototype.createNode for type: " + type);
  }
};

/**
 * Ignore default attributes.
 */
Node.prototype.specified = true;

/**
 * Add XPath support to nodes.
 */
Node.prototype.selectNodes = function(xpath)
{
  var xpe = new XPathEvaluator();
  var it;
  try
  {
    it = xpe.evaluate(xpath, this, xpe.createNSResolver(this), null, null);
  }
  catch (e)
  {
    __xopus_alert("Node.prototype.selectNodes('" + xpath + "')\ntrace:\n" + this.pathTrace + "\nxml:" + this.qxml());
  }

  var list = [], node;
  while (node = it.iterateNext())
  {
    if (!node.selectNodes)
    {
      var base;
      switch (node.nodeType)
      {
        case 1:
          base = Element;
          break;
        case 2:
          base = Attr;
          break;
        case 3:
          base = Text;
          break;
        case 4:
          base = CDataSection;
          break;
        default:
          __xopus_alert("Node.prototype.selectNodes unknown node type");
          return null;
      }

      for (var prop in base.prototype)
      {
        try {
          node[prop] = base.prototype[prop];
        } catch(e) {}
      }
    }

    if (node && node != this)
      node.pathTrace = (this.pathTrace ? this.pathTrace + "\n" : "") + "selectNodes('" + xpath + "')";

    list.push(node);
  }

  return list;
};

Node.prototype.selectSingleNode = function(xpath)
{
  var xpe = new XPathEvaluator();
  var result;
  try
  {
    result = xpe.evaluate(xpath, this, xpe.createNSResolver(this), XPathResult.FIRST_ORDERED_NODE_TYPE, null);
  }
  catch (e)
  {
    __xopus_alert("Node.prototype.selectSingleNode('" + xpath + "')\ntrace:\n" + this.pathTrace + "\nxml:" + this.qxml());
  }

  //check whether result
  if (!result) return null;

  var node = result.singleNodeValue;

  if (node && node != this)
    node.pathTrace = (this.pathTrace ? this.pathTrace + "\n" : "") + "selectSingleNode('" + xpath + "')";

  return node;
};

DocumentFragment.prototype.selectNodes = function(xpath)
{
  if (xpath.indexOf("::") >= xpath.indexOf("/"))
    xpath = "self::" + xpath;

  var childNodes = this.childNodes;
  var list = [];
  var xpe = new XPathEvaluator();
  for (var i = 0; i < childNodes.length; i++)
  {
    var n = childNodes[i];

    if (n.nodeType == 3)
      continue;

    var it = xpe.evaluate(xpath, n, xpe.createNSResolver(n), null, null);

    var node;
    while (node = it.iterateNext())
      list.push(node);
  }

  return list;
};

DocumentFragment.prototype.selectSingleNode = function(xpath)
{
  if (xpath.indexOf("::") >= xpath.indexOf("/"))
    xpath = "self::" + xpath;

  var childNodes = this.childNodes;
  var list = [];
  var xpe = new XPathEvaluator();
  for (var i = 0; i < childNodes.length; i++)
  {
    var n = childNodes[i];
    var result = n.selectSingleNode(xpath);

    if (result != null)
      return result;
  }

  return null;
};

/**
 * Add node serialization support.
 */
Node.prototype.__defineGetter__("xml", function()
{
  switch (this.nodeType)
  {
    case 1:
    case 9:
      return new XMLSerializer().serializeToString(this);
    case 2:
      return this.nodeName + "=" + this.nodeValue;
    case 3:
    case 4:
      return this.nodeValue;
    case 11:
      var res = "";
      for (var i = 0; i < this.childNodes.length; i++)
        res += this.childNodes[i].xml;
      return res;
    default:
      __xopus_alert("Type: " + this.nodeType + " not supported!");
  }
});

Node.prototype.qxml = function(indent)
{
  if (!indent)
    indent = "";

  switch (this.nodeType)
  {
    case 1:
      var str = indent + "<" + this.nodeName;

      for (var j = 0; j < this.attributes.length; j++)
        str += this.attributes[j].qxml();

      str += ">\n";

      for (var i = 0; i < this.childNodes.length; i++)
        str += this.childNodes[i].qxml(indent + "  ");

      str += indent + "</" + this.nodeName + ">\n";
      return str;
    case 2:
      return " " + this.nodeName + "=\"" + this.nodeValue + "\"";
    case 3:
      return indent + this.nodeValue + "\n";
    case 9:
      return this.documentElement.qxml();
    case 11:
      var str = "";
      for (var i = 0; i < this.childNodes.length; i++)
        str += this.childNodes[i].qxml() + "\n";
      return str;
  }
};

Node.prototype.__defineGetter__("text", function()
{
  var childNodes = this.childNodes;

  var str = "";
  for (var i = 0; i < childNodes.length; i++)
  {
    var n = childNodes[i];

    if (n.nodeType == 3)
      str += n.nodeValue;
    else
      str += n.text;
  }

  return str;
});

Node.prototype.__defineSetter__("text", function(text)
{
  var textNode = document.createTextNode(text);

  var childNodes = this.childNodes;

  for (var i = childNodes.length - 1; i >= 0; i--)
    this.removeChild(childNodes[i]);

  this.appendChild(textNode);
});

Text.prototype.__defineGetter__("text", function()
{
  return this.nodeValue;
});

Text.prototype.__defineSetter__("text", function(text)
{
  this.nodeValue = text;
});

Element.prototype.__q42_getAttribute = Element.prototype.getAttribute;
Element.prototype.getAttribute = function(name)
{
  var val = this.__q42_getAttribute(name);

  if (val == null)
    val = "";

  return val;
};

/**
 * Add XSLT support to nodes.
 */
Node.prototype.__q42_xslProcessor = new XSLTProcessor();
Node.prototype.transformNode = function(xslNode)
{
  var outputDoc = document.implementation.createDocument("text/xml", "", null);
  this.transformNodeToObject(xslNode, outputDoc);

  return outputDoc.xml;
};

Node.prototype.transformNodeToObject = function(xslNode, outputDoc)
{
  this.__q42_xslProcessor.transformDocument(this, xslNode, outputDoc, null);

  return outputDoc;
};

/**
 * Make the XSLTProcessor compatible with MSXML2.XSLTProcessor
 */
XSLTProcessor.prototype.__defineSetter__("stylesheet", function(stylesheet)
{
  this.__q42_stylesheet = stylesheet;
});

XSLTProcessor.prototype.addParameter = function(name, value)
{
  if (this.__q42_stylesheet)
  {
    var paramNode = this.__q42_stylesheet.selectSingleNode("//*[local-name()='param'][namespace-uri()='http://www.w3.org/1999/XSL/Transform'][@name='" + name + "']");

    var childNodes = paramNode.childNodes;
    for (var i = childNodes.length - 1; i >= 0; i--)
      paramNode.removeChild(childNodes.item(i));

    var textNode = this.__q42_stylesheet.createTextNode(value);
    paramNode.appendChild(textNode);
  }
};

XSLTProcessor.prototype.__defineSetter__("input", function(inputDoc)
{
  this.__q42_inputDoc = inputDoc;
});

XSLTProcessor.prototype.__defineSetter__("output", function(outputDoc)
{
  this.__q42_outputDoc = outputDoc;
});

XSLTProcessor.prototype.__defineGetter__("output", function()
{
  return this.__q42_outputDoc.xml;
});

XSLTProcessor.prototype.transform = function()
{
  if (!this.__q42_outputDoc)
    this.__q42_outputDoc = document.implementation.createDocument("text/xml", "", null);
  this.__q42_inputDoc.transformNodeToObject(this.__q42_stylesheet, this.__q42_outputDoc);
};

/**
 * Add IE specific DHTML support.
 */
HTMLElement.prototype.getBoundingClientRect = function()
{
  var rect = { left: 0, top: 0, right: 0, bottom: 0 };

  var tempElt = this;
  while (tempElt)
  {
    rect.top  += tempElt.offsetTop - tempElt.scrollTop;
    rect.left += tempElt.offsetLeft - tempElt.scrollLeft;
    tempElt = tempElt.offsetParent;
  }

  rect.bottom = rect.top + this.offsetHeight;
  rect.right  = rect.left + this.offsetWidth;

  return rect;
};

HTMLElement.prototype.__defineGetter__("children", function()
{
  return this.selectNodes("*");
});

HTMLElement.prototype.__defineGetter__("innerText", function()
{
  var t = '';
  for (var i=0; i<this.childNodes.length; i++) {
    var child = this.childNodes[i];
    switch (child.nodeType) {
      case 1:
        t += child.innerText;
        break;
      case 3:
        t += child.nodeValue;
        break;
    }
  }
  return t;
});

HTMLElement.prototype.__defineSetter__("innerText", function(newText)
{
  var newTextNode = document.createTextNode(newText);
  var children = this.childNodes;

  for (var i = children.length - 1; i >= 0; i--)
    this.removeChild(children[i]);

  this.appendChild(newTextNode);
});

HTMLElement.prototype.__defineGetter__("XMLDocument", function()
{
  var dp = new DOMParser();
  var xmlDOM = dp.parseFromString(this.innerHTML, "text/xml");
  return xmlDOM;
});

HTMLElement.prototype.__defineSetter__("unselectable", function(value)
{
  if (value == "on") {
    this.style.MozUserSelect = "none";
    this.style.MozUserFocus = "ignore";
  } else {
    this.style.MozUserSelect = "text";
    this.style.MozUserFocus = "normal";
  }
});

navigator.userLanguage = navigator.language;
navigator.systemLanguage = navigator.language;
navigator.browserLanguage = navigator.language;

/**
 * Implement current style
 */
HTMLElement.prototype.__defineGetter__("currentStyle", function()
{
  return new __q42_ComputedStyle(document.defaultView.getComputedStyle(this, ""));
});

function __q42_ComputedStyle(base)
{
  this.base = base;
}

__q42_ComputedStyle.prototype.__defineGetter__("display", function()
{
  return this.base.getPropertyValue("display");
});

__q42_ComputedStyle.prototype.__defineGetter__("zIndex", function()
{
  return this.base.getPropertyValue("z-index");
});

/**
 * Implement document.selection
 */
HTMLDocument.prototype.__defineGetter__("selection", function()
{
  return new __q42_Selection(window.getSelection());
});

function __q42_Selection(mozSelection)
{
  this.mozSelection = mozSelection;
}

__q42_Selection.prototype.createRange = function()
{
  if (this.mozSelection.rangeCount > 0) {
    return new __q42_TextRange(this.mozSelection.getRangeAt(0));
  } else
    return null;
};

HTMLBodyElement.prototype.createTextRange = function()
{
  if (window.getSelection().rangeCount > 0) {
    return new __q42_TextRange(window.getSelection().getRangeAt(0));
  } else
    return null;
};

__q42_Selection.prototype.empty = function()
{
  window.getSelection().removeAllRanges();
};

__q42_Selection.prototype.__defineGetter__("type", function()
{
  if (this.mozSelection.rangeCount == 0)
    return "None";
  else
    return "Text";
});

function __q42_TextRange(mozDOMRange)
{
  this.mozDOMRange = mozDOMRange;
}

__q42_TextRange.prototype.parentElement = function()
{
  var parent = this.mozDOMRange.commonAncestorContainer;

  if (parent.nodeType == 3)
    return parent.parentNode;
  else
    return parent;
};

__q42_TextRange.prototype.__defineGetter__("htmlText", function()
{
  return this.mozDOMRange.cloneContents().xml;
});

__q42_TextRange.prototype.__defineGetter__("text", function()
{
  return this.mozDOMRange.cloneContents().text;
});

__q42_TextRange.prototype.__defineSetter__("text", function(text)
{
  // Remove content of current selection from document
  this.mozDOMRange.deleteContents();

  // Get location of current selection
  var container = this.mozDOMRange.startContainer;
  var pos       = this.mozDOMRange.startOffset;

  if (container.nodeType == 3)
  {
    container.insertData(pos, text);
  }
  else
  {
    var doc = this.mozDOMRange.commonAncestorContainer.ownerDocument;
    var insertNode = doc.createTextNode(text);
    var afterNode  = container.childNodes[pos];
    container.insertBefore(insertNode, afterNode);
  }
});

__q42_TextRange.prototype.duplicate = function()
{
  return new __q42_TextRange(this.mozDOMRange.cloneRange());
};

__q42_TextRange.prototype.collapse = function(toStart)
{
  //this.mozDOMRange.collapse(toStart);
  var container; var offset;
  if (toStart)
  {
    container = this.mozDOMRange.startContainer;
    offset = this.mozDOMRange.startOffset;
  }
  else
  {
    container = this.mozDOMRange.endContainer;
    offset = this.mozDOMRange.endOffset;
  }

  this.mozDOMRange = document.createRange();
  this.mozDOMRange.setEnd(container, offset);
  this.mozDOMRange.setStart(container, offset);
};

__q42_TextRange.prototype.expand = function(unit, count)
{
  var startPoint = {node: this.mozDOMRange.startContainer, offset: this.mozDOMRange.startOffset};
  var endPoint   = {node: this.mozDOMRange.endContainer, offset: this.mozDOMRange.endOffset};

  if (count === undefined)
    count = 1;

  for (;count > 0; count--)
  {
    switch (unit.toLowerCase())
    {
      case "character":
        var point = this.nextChar(endPoint);
        this.mozDOMRange.setEnd(point.node, point.offset);
        break;
      case "word":
        var newStartPoint = this.findStartOfUnit (startPoint, /\s|\u00A0/);
        var newEndPoint   = this.findEndOfUnit (endPoint, /\s|\u00A0/);

        this.mozDOMRange = document.createRange();
        this.mozDOMRange.setEnd(newEndPoint.node, newEndPoint.offset);
        this.mozDOMRange.setStart(newStartPoint.node, newStartPoint.offset);
        break;
      case "sentence":
        var newStartPoint = this.findStartOfUnit (startPoint, /[.,;:]/);
        var newEndPoint   = this.findEndOfUnit (endPoint, /[.,;:]/);

        this.mozDOMRange = document.createRange();
        this.mozDOMRange.setEnd(newEndPoint.node, newEndPoint.offset);
        this.mozDOMRange.setStart(newStartPoint.node, newStartPoint.offset);
        break;
      case "textedit":
        break;
    }
  }
};

// TODO replace by something readable
__q42_TextRange.prototype.findEndOfUnit = function (point, regExp)
{
  // TODO: this tests point while findStartOfUnit tests prevPoint
  while (!regExp.test(point.node.nodeValue.substr(point.offset, 1)))
  {
    var nextPoint = this.nextChar(point);

    if (nextPoint)
      point = nextPoint;
    else
      break;
  }

  return point;
};

// TODO replace by something readable
__q42_TextRange.prototype.findStartOfUnit = function (point, regExp)
{
  var prevPoint = point;
  // TODO: this tests prevPoint while findStartOfUnit tests point
  while (prevPoint && !regExp.test(prevPoint.node.nodeValue.substr(prevPoint.offset, 1)))
  {
    point = prevPoint;
    prevPoint = this.prevChar(point);
  }

  return point;
};

// TODO replace by something readable
__q42_TextRange.prototype.nextChar = function (point)
{
  // Don't expand beyond contentEditable boundaries.
  if (point.node.nodeType == 1 && point.node.contentEditable)
    return null;

  if (point.offset == this.nodeLength(point.node))
    return this.nextChar({node: point.node.parentNode, offset: xopus_globs.utils.indexOf(point.node) + 1});

  switch (point.node.nodeType)
  {
    case 1:
      if (point.offset == -1)
        point.offset++;

      return this.nextChar({node: point.node.childNodes[point.offset], offset: -1});
    case 3:
      return {node: point.node, offset: point.offset + 1};
    default:
      __xopus_alert("Not implemented: __q42_TextRange.prototype.nextChar node=" + point.node.xml);
  }
};

// TODO replace by something readable
__q42_TextRange.prototype.prevChar = function (point)
{
  // Don't expand beyond contentEditable boundaries.
  if (point.node.nodeType == 1 && point.node.contentEditable)
    return null;

  switch (point.node.nodeType)
  {
    case 1:
      if (point.offset == point.node.childNodes.length)
        point.offset--;

      if (point.offset == -1)
        return this.prevChar({node: point.node.parentNode, offset: xopus_globs.utils.indexOf(point.node) - 1});
      else
        return this.prevChar({node: point.node.childNodes[point.offset], offset: this.nodeLength(point.node.childNodes[point.offset])});
    case 3:
      if (point.offset == 0)
        return this.prevChar({node: point.node.parentNode, offset: xopus_globs.utils.indexOf(point.node) - 1});
      else
        return {node: point.node, offset: point.offset - 1};
    default:
      __xopus_alert("Not implemented: __q42_TextRange.prototype.nextChar node=" + point.node.xml);
  }
};

__q42_TextRange.prototype.nodeLength = function (node)
{
  switch (node.nodeType)
  {
    case 1:
      return node.childNodes.length;
    case 3:
      return node.nodeValue.length;
  }
};

__q42_TextRange.prototype.moveToElementText = function(node)
{
  var first = this.findFirstTextNode(node);
  var last = this.findLastTextNode(node);

  this.mozDOMRange = document.createRange();
  this.mozDOMRange.setEnd(last, last.nodeValue.length);
  this.mozDOMRange.setStart(first, 0);
};

__q42_TextRange.prototype.findFirstTextNode = function(node)
{
  switch (node.nodeType)
  {
    case 1:
    case 9:
    case 11:
      if (node.childNodes.length > 0)
        return this.findFirstTextNode(node.firstChild);
      else
        return this.findFirstTextNode(node.nextSibling);
    case 3:
      return node;
    default:
      __xopus_alert("Not implemented: __q42_TextRange.prototype.findFirstTextNode type=" + node.nodeType);
  }
};

__q42_TextRange.prototype.findLastTextNode = function(node)
{
  switch (node.nodeType)
  {
    case 1:
    case 9:
    case 11:
      if (node.childNodes.length > 0)
        return this.findLastTextNode(node.lastChild);
      else
        return this.findLastTextNode(node.previousSibling);
    case 3:
      return node;
    default:
      __xopus_alert("Not implemented: __q42_TextRange.prototype.findLastTextNode type=" + node.nodeType);
  }
};

__q42_TextRange.prototype.setEndPoint = function(type, range)
{
  try {
    switch (type.toLowerCase())
    {
      case "starttoend":
        this.mozDOMRange.setStart(range.mozDOMRange.endContainer, range.mozDOMRange.endOffset);
        break;
      case "starttostart":
        this.mozDOMRange.setStart(range.mozDOMRange.startContainer, range.mozDOMRange.startOffset);
        break;
      case "endtostart":
        this.mozDOMRange.setEnd(range.mozDOMRange.startContainer, range.mozDOMRange.startOffset);
        break;
      case "endtoend":
        this.mozDOMRange.setEnd(range.mozDOMRange.endContainer, range.mozDOMRange.endOffset);
        break;
    }
  } catch(e) {
//TODO: BUG: in moz, click in editor, select text, Ctrl+B, click in newly created bold part: error
//    __xopus_alert(range.mozDOMRange.startContainer + ', ' + range.mozDOMRange.startOffset + ', ' + e.message);
  }
};

__q42_TextRange.prototype.moveStart = function(type, offset)
{
  switch (type.toLowerCase())
  {
    case "character":
      var point = this.getNodeForOffset(this.mozDOMRange.startContainer, this.mozDOMRange.startOffset, offset);
      this.mozDOMRange.setStart(point.node, point.offset);
      break;
    default:
      __xopus_alert("Not implemented: __q42_TextRange.prototype.moveStart for type equals " + type);
      break;
  }
};

__q42_TextRange.prototype.moveEnd = function(type, offset)
{
  switch (type.toLowerCase())
  {
    case "character":
        //TODO: this sometimes fails!
      var point = this.getNodeForOffset(this.mozDOMRange.endContainer, this.mozDOMRange.endOffset, offset);
      this.mozDOMRange.setEnd(point.node, point.offset);
      break;
    default:
      __xopus_alert("Not implemented: __q42_TextRange.prototype.moveEnd for type equals " + type);
      break;
  }
};

__q42_TextRange.prototype.getNodeForOffset = function(startNode, startOffset, offset)
{
  switch (startNode.nodeType)
  {
    case 1:
      if (startOffset == startNode.childNodes.length)
        return this.getNodeForOffset(startNode.parentNode, xopus_globs.utils.indexOf(startNode) + 1, offset);
      else
        return this.getNodeForOffset(startNode.childNodes[startOffset], 0, offset);
    case 3:
      if (startNode.nodeValue.length >= startOffset + offset)
        return {node: startNode, offset: startOffset + offset};
      else
        return this.getNodeForOffset(startNode.parentNode, xopus_globs.utils.indexOf(startNode) + 1, offset - startNode.nodeValue.length);
    default:
      __xopus_alert("Not implemented: __q42_TextRange.prototype.getNodeForOffset node=" + startNode.xml);
  }
};

__q42_TextRange.prototype.moveEnd = function(type, offset)
{
  switch (type.toLowerCase())
  {
    case "character":
      this.mozDOMRange.setEnd(this.mozDOMRange.endContainer, offset);
      break;
    default:
      __xopus_alert("Not implemented: __q42_TextRange.prototype.moveStart");
      break;
  }
};

__q42_TextRange.prototype.select = function()
{
  var sel = window.getSelection();
  sel.removeAllRanges();
  sel.addRange(this.mozDOMRange);
};

__q42_TextRange.prototype.isEqual = function(rhs)
{
  return this.mozDOMRange.startContainer == rhs.mozDOMRange.startContainer &&
         this.mozDOMRange.startOffset == rhs.mozDOMRange.startOffset &&
         this.mozDOMRange.endContainer == rhs.mozDOMRange.endContainer &&
         this.mozDOMRange.endOffset == rhs.mozDOMRange.endOffset;
};

__q42_TextRange.prototype.compareEndPoints = function(type, range)
{
  var how;
  switch (type.toLowerCase())
  {
    case "starttoend":
      how=this.mozDOMRange.START_TO_END;
      break;
    case "starttostart":
      how=this.mozDOMRange.START_TO_START;
      break;
    case "endtostart":
      how=this.mozDOMRange.END_TO_START;
      break;
    case "endtoend":
      how=this.mozDOMRange.END_TO_END;
      break;
  }
  return this.mozDOMRange.compareBoundaryPoints(how, range.mozDOMRange);
};

HTMLDocument.prototype.queryCommandEnabled = function(id)
{
  return true;
};

/**
 * Add contentEditable property and basic editing functions.
 */
HTMLElement.prototype.__defineSetter__("contentEditable", function(value)
{
  value = "" + value == "true";
  this.__isContentEditable = value;
  this.setAttribute('contentEditable', '' + value);
  if (value)
  {
    if (!document.hasEditableElement)
    {
      document.body.style.MozUserInput = "enabled";
      document.body.style.MozUserFocus = "normal";
      document.hasEditableElement = true;
    }
  }
});

HTMLElement.prototype.__defineGetter__("contentEditable", function()
{
  return this.__isContentEditable;
});

HTMLElement.prototype.__defineGetter__("isContentEditable", function()
{
  var el = this;
  while (el) {
    if (el.__isContentEditable == true) return true;
    else el = el.parentNode;
  }
  return false;
});

HTMLDocument.prototype.__q42_keypress = function(evt, theWindow)
{
//__xopus_alert('keypress = ' + evt.metaKey + ', ' + evt.ctrlKey + ', ' + evt.altKey + ', ' + evt.keyCode + ', ' + evt.charCode);
  // don't do anything when the alt or meta key is pressed
  if (evt.altKey || evt.ctrlKey || evt.metaKey) return true;

  // get current selection
  var sel = theWindow.getSelection();

  // get the first range of the selection
  // (there's almost always only one range)
  try {
    var range = sel.getRangeAt(0);
  } catch(e) {
    return true;
  }

  var eventTarget = range.commonAncestorContainer;
  if (eventTarget.nodeType == 3) eventTarget = eventTarget.parentNode;

  //check whether inside content editable part
  if (!eventTarget.isContentEditable) return true;

  // current selection will be replaced with insertNode if set
  var insertNode = null;

  switch (evt.keyCode)
  {
    case evt.DOM_VK_HOME:
      // home key pressed
      // collapse selection to start of contentEditable element
      sel.removeAllRanges();
      range.selectNodeContents(this);
      range.collapse(true);
      sel.addRange(range);
      evt.preventDefault();
      return false;
    case evt.DOM_VK_END:
      // end key pressed
      // collapse selection to end of contentEditable element
      sel.removeAllRanges();
      range.selectNodeContents(this);
      range.collapse(false);
      sel.addRange(range);
      evt.preventDefault();
      return false;
    case evt.DOM_VK_RETURN:
    case evt.DOM_VK_ENTER:
      // return key pressed
      // insert break
      //insertNode = document.createElement('br');
      return false;
      break;
    case evt.DOM_VK_DELETE:
      // delete key pressed
      if (range.collapsed)
      {
        // if no selection, select next character
        try
        {
          range.setEnd(range.endContainer, range.endOffset+1);
        }
        catch(e){}
      }
      range.deleteContents();
      return false;
      break;
    case evt.DOM_VK_BACK_SPACE:
      // backspace key pressed
      if (range.collapsed)
      {
        // if no selection, select previous character
        try
        {
          range.setStart(range.startContainer, range.startOffset-1);
        }
        catch(e){}
      }
      range.deleteContents();
      return false;
      break;
    default:
      // normal key pressed
      if (evt.charCode) {
        insertNode = document.createTextNode(String.fromCharCode(evt.charCode));
      } else {
        return true;
      }
      break;
  }

  if (insertNode)
  {
    this.__q42_insertNodeAtSelection(insertNode);
    evt.preventDefault();
  }
  return true;
};

HTMLDocument.prototype.__q42_insertNodeAtSelection = function(insertNode)
{
  // get current selection
  var sel = window.getSelection();

  // get the first range of the selection
  // (there's almost always only one range)
  var range = sel.getRangeAt(0);

  // deselect everything
  sel.removeAllRanges();

  // remove content of current selection from document
  range.deleteContents();

  // get location of current selection
  var container = range.startContainer;
  var pos = range.startOffset;

  // make a new range for the new selection
  range=document.createRange();

  if (container.nodeType==3&&insertNode.nodeType==3) {

    // if we insert text in a textnode, do optimized insertion
    container.insertData(pos, insertNode.nodeValue);

    // put cursor after inserted text
    range.setEnd(container, pos+insertNode.length);
    range.setStart(container, pos+insertNode.length);

  } else {


    var afterNode;
    if (container.nodeType==3) {

      // when inserting into a textnode
      // we create 2 new textnodes
      // and put the insertNode in between

      var textNode = container;
      container = textNode.parentNode;
      var text = textNode.nodeValue;

      // text before the split
      var textBefore = text.substr(0,pos);
      // text after the split
      var textAfter = text.substr(pos);

      var beforeNode = document.createTextNode(textBefore);
      var afterNode = document.createTextNode(textAfter);

      // insert the 3 new nodes before the old one
      container.insertBefore(afterNode, textNode);
      container.insertBefore(insertNode, afterNode);
      container.insertBefore(beforeNode, insertNode);

      // remove the old node
      container.removeChild(textNode);

    } else {

      // else simply insert the node
      afterNode = container.childNodes[pos];
      container.insertBefore(insertNode, afterNode);
    }

    range.setEnd(afterNode, 0);
    range.setStart(afterNode, 0);
  }

  sel.addRange(range);
};

try {
  numScriptsLoading--;
} catch(e) {}

