<!--

  This stylesheet marks all element, attributes and text nodes with processing instructions
  to help Xopus find these nodes back in the original XML after an XSL transformation.

  Attribute encoding

    <elementName attributeName="value">
      text
    </elementName>

    Encoded:

    <elementName attributeName="value">
      <?xopus_attribute attributeName=/elementName[1]/@attributeName?>
      text
    </elementName>

  Element encoding

    <elementName attributeName="value">
      text
    </elementName>

    Encoded:

    <elementName attributeName="value">
      text
    </elementName>
    <?xopus_element elementName=/elementName[1]?>

  Textnode encoding

    <elementName attributeName="value">
      text
    </elementName>

    Encoded:

    <elementName attributeName="value">
      <?xopus_separator?>text<?xopus_text /elementName[1]/text()[1]?>
    </elementName>  

-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  
  <xsl:template match="/">
    <container>
      <xsl:choose>
        <xsl:when test="name() = ''">
          <xsl:for-each select="*">
            <xsl:call-template name="elt"/>
          </xsl:for-each>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="elt"/>
        </xsl:otherwise>
      </xsl:choose>
    </container>
  </xsl:template>
  
  <xsl:template match="text()">
    <container>
      <xsl:call-template name="txt"/>
    </container>
  </xsl:template>
  
  <xsl:template match="*">
    <container>
      <xsl:call-template name="elt"/>
    </container>
  </xsl:template>
  
  <xsl:template match="text()" mode="child">
    <xsl:call-template name="txt"/>
  </xsl:template>
  
  <xsl:template match="*" mode="child">
    <xsl:call-template name="elt"/>
  </xsl:template>
  
  <xsl:template name="elt">
    <xsl:variable name="xpath">//*[@*[local-name() = 'id' and namespace-uri() = 'http://www.xopus.org/xmlns/id'] = '<xsl:value-of select="string(@*[local-name() = 'id' and namespace-uri() = 'http://www.xopus.org/xmlns/id'])"/>']</xsl:variable>
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:for-each select="@*">
        <xsl:if test="not(namespace-uri() = 'http://www.xopus.org/xmlns/id')">
          <xsl:processing-instruction name="xopus_attribute"><xsl:value-of select="name()"/>=<xsl:value-of select="$xpath"/>/@*[local-name() = '<xsl:value-of select="string(local-name())"/>' and namespace-uri()='<xsl:value-of select="string(namespace-uri())"/>']</xsl:processing-instruction>
        </xsl:if>
      </xsl:for-each>
      <xsl:apply-templates select="*|text()" mode="child"><xsl:with-param name="xpath"><xsl:value-of select="$xpath"/></xsl:with-param></xsl:apply-templates>
    </xsl:copy>
    <xsl:processing-instruction name="xopus_element"><xsl:value-of select="name()"/>=<xsl:value-of select="$xpath"/></xsl:processing-instruction>    
  </xsl:template>
  
  <xsl:template name="txt">
    <xsl:param name="xpath">//*[@*[local-name() = 'id' and namespace-uri() = 'http://www.xopus.org/xmlns/id'] = '<xsl:value-of select="string(../@*[local-name() = 'id' and namespace-uri() = 'http://www.xopus.org/xmlns/id'])"/>']</xsl:param>
    <xsl:processing-instruction name="xopus_separator"/>
    <xsl:value-of select="."/> 
    <xsl:processing-instruction name="xopus_text"><xsl:value-of select="$xpath"/>/text()[<xsl:value-of select="number(count(preceding-sibling::text()) + 1)"/>]</xsl:processing-instruction>
  </xsl:template>
</xsl:stylesheet>