<!-- This stylesheet alters xsl:value-of and xsl:copy-of elements in such a way that they pass our processing instructions on -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:template match="*|@*|text()"><xsl:copy><xsl:apply-templates select="*|@*|text()" /></xsl:copy></xsl:template>
  
  <!-- Add templates to create an xpath for the given node. -->
  <xsl:template match="xsl:stylesheet">
    <xsl:copy>
      <!-- Copy attributes -->
      <xsl:apply-templates select="@*"/>
      
      <!-- Include the XPath generator -->
      <xsl:element name="xsl:include">
        <xsl:attribute name="href">xopus/pipeline/xpath.xsl</xsl:attribute>
        <xsl:attribute name="xopus">true</xsl:attribute>
      </xsl:element>
    
      <xsl:apply-templates select="*|text()"/>
      
      <!-- Handle xsl:copy -->
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">*</xsl:attribute>
        <xsl:attribute name="mode">copy</xsl:attribute>
        <xsl:element name="xsl:param">
          <xsl:attribute name="name">orgCopy</xsl:attribute>
        </xsl:element>
        <xsl:element name="xsl:copy-of">
          <xsl:attribute name="select">$orgCopy</xsl:attribute>
        </xsl:element>
        <xsl:element name="xsl:copy-of">
          <xsl:attribute name="select">following-sibling::processing-instruction("xopus_element")[substring-before(., "=") = name(current())][1]</xsl:attribute>
        </xsl:element>
      </xsl:element>
        
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">@*</xsl:attribute>
        <xsl:attribute name="mode">copy</xsl:attribute>
        <xsl:element name="xsl:param">
          <xsl:attribute name="name">orgCopy</xsl:attribute>
        </xsl:element>
        <xsl:element name="xsl:copy"/>
        <xsl:element name="xsl:if">
          <xsl:attribute name="test">../processing-instruction("xopus_attribute")[substring-before(., "=") = name(current())]</xsl:attribute>
          <xsl:element name="xsl:copy-of">
            <xsl:attribute name="select">../processing-instruction("xopus_attribute")[substring-before(., "=") = name(current())]</xsl:attribute>
          </xsl:element>
        </xsl:element>
      </xsl:element>
        
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">text()</xsl:attribute>
        <xsl:attribute name="mode">copy</xsl:attribute>
        <xsl:element name="xsl:param">
          <xsl:attribute name="name">orgCopy</xsl:attribute>
        </xsl:element>
        <xsl:element name="xsl:choose">
          <xsl:element name="xsl:when">
            <xsl:attribute name="test">name(preceding-sibling::processing-instruction()[1]) = 'xopus_separator' and name(following-sibling::processing-instruction()[1]) = 'xopus_text'</xsl:attribute>
            <xsl:element name="xsl:copy-of">
              <xsl:attribute name="select">preceding-sibling::processing-instruction('xopus_separator')[1]</xsl:attribute>
            </xsl:element>
            <xsl:element name="xsl:copy-of">
              <xsl:attribute name="select">$orgCopy</xsl:attribute>
            </xsl:element>
            <xsl:element name="xsl:copy-of">
              <xsl:attribute name="select">following-sibling::processing-instruction('xopus_text')[1]</xsl:attribute>
            </xsl:element>
          </xsl:element>
          <xsl:element name="xsl:otherwise">
            <xsl:element name="xsl:copy-of">
              <xsl:attribute name="select">$orgCopy</xsl:attribute>
            </xsl:element>
          </xsl:element>
        </xsl:element>
      </xsl:element>
        
      <!-- Handle xsl:copy-of -->
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">*</xsl:attribute>
        <xsl:attribute name="mode">copyOf</xsl:attribute>
        <xsl:element name="xsl:copy-of">
          <xsl:attribute name="select">.</xsl:attribute>
        </xsl:element>
        <xsl:element name="xsl:copy-of">
          <xsl:attribute name="select">following-sibling::processing-instruction("xopus_element")[substring-before(., "=") = name(current())][1]</xsl:attribute>
        </xsl:element>
      </xsl:element>
        
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">@*</xsl:attribute>
        <xsl:attribute name="mode">copyOf</xsl:attribute>
        <xsl:element name="xsl:copy-of">
          <xsl:attribute name="select">.</xsl:attribute>
        </xsl:element>
        <xsl:element name="xsl:if">
          <xsl:attribute name="test">../processing-instruction("xopus_attribute")[substring-before(., "=") = name(current())]</xsl:attribute>
          <xsl:element name="xsl:copy-of">
            <xsl:attribute name="select">../processing-instruction("xopus_attribute")[substring-before(., "=") = name(current())]</xsl:attribute>
          </xsl:element>
        </xsl:element>
      </xsl:element>
        
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">text()</xsl:attribute>
        <xsl:attribute name="mode">copyOf</xsl:attribute>
        <xsl:element name="xsl:choose">
          <xsl:element name="xsl:when">
            <xsl:attribute name="test">name(preceding-sibling::processing-instruction()[1]) = 'xopus_separator' and name(following-sibling::processing-instruction()[1]) = 'xopus_text'</xsl:attribute>
            <xsl:element name="xsl:copy-of">
              <xsl:attribute name="select">preceding-sibling::processing-instruction('xopus_separator')[1]</xsl:attribute>
            </xsl:element>
            <xsl:element name="xsl:copy-of">
              <xsl:attribute name="select">.</xsl:attribute>
            </xsl:element>
            <xsl:element name="xsl:copy-of">
              <xsl:attribute name="select">following-sibling::processing-instruction('xopus_text')[1]</xsl:attribute>
            </xsl:element>
          </xsl:element>
          <xsl:element name="xsl:otherwise">
            <xsl:element name="xsl:copy-of">
              <xsl:attribute name="select">.</xsl:attribute>
            </xsl:element>            
          </xsl:element>
        </xsl:element>
      </xsl:element>
        
      <!-- Handle xsl:value-of -->
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">*</xsl:attribute>
        <xsl:attribute name="mode">valueOf</xsl:attribute>
        <xsl:element name="xsl:apply-templates">
          <xsl:attribute name="select">*|text()</xsl:attribute>
          <xsl:attribute name="mode">valueOf</xsl:attribute>
        </xsl:element>
      </xsl:element>
        
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">@*</xsl:attribute>
        <xsl:attribute name="mode">valueOf</xsl:attribute>
        <xsl:element name="xsl:choose">
          <xsl:element name="xsl:when">
            <xsl:attribute name="test">../processing-instruction("xopus_attribute")[substring-before(.,"=") = name(current())]</xsl:attribute>
            <xsl:element name="xsl:processing-instruction">
              <xsl:attribute name="name">xopus_separator</xsl:attribute>
            </xsl:element>
            <xsl:element name="xsl:value-of">
              <xsl:attribute name="select">.</xsl:attribute>
            </xsl:element>
            <xsl:element name="xsl:processing-instruction">
              <xsl:attribute name="name">xopus_text</xsl:attribute>
              <xsl:element name="xsl:value-of">
                <xsl:attribute name="select">substring-after(../processing-instruction("xopus_attribute")[substring-before(.,"=") = name(current())][1], "=")</xsl:attribute>
              </xsl:element>
            </xsl:element>
          </xsl:element>
          <xsl:element name="xsl:otherwise">
            <xsl:element name="xsl:value-of">
              <xsl:attribute name="select">.</xsl:attribute>
            </xsl:element>
          </xsl:element>
        </xsl:element>
      </xsl:element>
        
      <xsl:element name="xsl:template">
        <xsl:attribute name="match">text()</xsl:attribute>
        <xsl:attribute name="mode">valueOf</xsl:attribute>
        <xsl:element name="xsl:choose">
          <xsl:element name="xsl:when">
            <xsl:attribute name="test">name(preceding-sibling::processing-instruction()[1]) = 'xopus_separator' and name(following-sibling::processing-instruction()[1]) = 'xopus_text'</xsl:attribute>
            <xsl:element name="xsl:processing-instruction">
              <xsl:attribute name="name">xopus_separator</xsl:attribute>
            </xsl:element>
            <xsl:element name="xsl:value-of">
              <xsl:attribute name="select">.</xsl:attribute>
            </xsl:element>
            <xsl:element name="xsl:processing-instruction">
              <xsl:attribute name="name">xopus_text</xsl:attribute>
              <xsl:element name="xsl:value-of">
                <xsl:attribute name="select">following-sibling::processing-instruction('xopus_text')[1]</xsl:attribute>
              </xsl:element>
            </xsl:element>
          </xsl:element>
          <xsl:element name="xsl:otherwise">
            <xsl:element name="xsl:value-of">
              <xsl:attribute name="select">.</xsl:attribute>
            </xsl:element>            
          </xsl:element>
        </xsl:element>
      </xsl:element>
    </xsl:copy>
  </xsl:template>
</xsl:stylesheet>