<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:xs="http://www.w3.org/2001/XMLSchema">

<xsl:output indent="yes" />

<xsl:template match="/">
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified">
	<xsl:apply-templates select="data/ElementType" />
</xs:schema>
</xsl:template>

<!-- Elements which are NOT textholding elements but holds other elements -->
<xsl:template match="ElementType[@containsnodes='true' or position()=1]">
<!--<xsl:template match="ElementType[@containsnodes='true' and @text='false' and @integer='false']">-->
	<xs:element name="{@name}">
		<xs:complexType>
			<xsl:if test="@containsnodes='true' or @text='true'"><xsl:attribute name="mixed">true</xsl:attribute></xsl:if>
			<xsl:if test="/*/ElementType/parent[@name=current()/@name]">
			<xs:sequence><xs:choice minOccurs="0" maxOccurs="unbounded">
				<xsl:apply-templates select="/*/ElementType/parent[@name=current()/@name]" />
			</xs:choice></xs:sequence>
			</xsl:if>
		<xsl:apply-templates select="//AttributeType/parent[@name=current()/@name]" />
		</xs:complexType>
	</xs:element><xsl:text>
	</xsl:text>
</xsl:template>

<xsl:template match="ElementType[@containsnodes='false' and position()&gt;1]">
</xsl:template>

<!-- elements with no children -->
<xsl:template match="ElementType/parent[../@containsnodes='false']">
	<xsl:variable name="name" select="../@name" />
	<xsl:variable name="attrs" select="//AttributeType/parent[@name=$name]" />

	<!-- with attributes -->
	<xsl:if test="$attrs">
		<xs:element name="{$name}" minOccurs="{@minOccurs}" maxOccurs="{@maxOccurs}">
		<xs:complexType>
		<xsl:if test="@containsnodes='true' or @text='true'"><xsl:attribute name="mixed">true</xsl:attribute></xsl:if>
			<xsl:apply-templates select="$attrs" />
		</xs:complexType>
		</xs:element>
	</xsl:if>

	<!-- without attributes -->	
	<xsl:if test="not($attrs)">
		<xs:element name="{$name}" type="xs:string" minOccurs="{@minOccurs}" maxOccurs="{@maxOccurs}" />
	</xsl:if>

	<xsl:text>
	</xsl:text>
</xsl:template>

<!-- elements with children -->
<xsl:template match="ElementType/parent[../@containsnodes='true']">
	<xsl:variable name="name" select="../@name" />
	<xs:element ref="{$name}" minOccurs="{@minOccurs}" maxOccurs="{@maxOccurs}"></xs:element><xsl:text>
	</xsl:text>
</xsl:template>

<!-- Attribute -->
<xsl:template match="AttributeType/parent">
	<xs:attribute name="{../@name}">
		<xsl:if test="@minOccurs='1'"><xsl:attribute name="use">required</xsl:attribute></xsl:if>
		<xsl:attribute name="type">
			<xsl:choose>
				<xsl:when test="../@integer='true' and ../@text='false'">xs:int</xsl:when>
				<xsl:otherwise>xs:string</xsl:otherwise>
			</xsl:choose>	
		</xsl:attribute>
	</xs:attribute>
</xsl:template>

</xsl:stylesheet>
