////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

//Schema Builder
//
// derives a basic-schema when given an XML DOM for input

//Constructor
function SchemaBuilder() {
  var o = new Object();

  //property
  o.xslt = null;

  //methods
  o.process = process_schemabuilder;
  o.learnDistinct = learnDistinct_schemabuilder;
  o.countOccurences = countOccurences_schemabuilder;
  o.learnParents = learnParents_schemabuilder;
  o.calculateProperties = calculateProperties_schemabuilder;
  o.calculateProperties = calculateProperties_schemabuilder;
  o.learnValues = learnValues_schemabuilder;
  o.learnCounters = learnCounters_schemabuilder;

  return o;
}

function process_schemabuilder(xml) {
  // process xml and learn from it. Knowledge is stored in data xml

  //load xslt if not present
  if (!this.xslt) this.xslt = xmllib.load("buildschema.xsl");

  //create the data xml storage
  var data = xmllib.parse("<data/>");

  //get all distinct attributes
  var attrs = xml.selectNodes("//*/@*");
  var attrBeenThere = new Array();
  this.learnDistinct(data.documentElement, attrs, "AttributeType");

  //get all distinct nodes
  var nodes = xml.selectNodes("//*");
  this.learnDistinct(data.documentElement, nodes, "ElementType");

  //investigate parents
  var nodes = xml.selectNodes("//*");
//__xopus_alert(data.xml);
  this.learnParents(data.documentElement, nodes, "ElementType");
//__xopus_alert(data.xml);

  var attrs = xml.selectNodes("//@*");
  this.learnParents(data.documentElement, attrs, "AttributeType");

  //investigate values
  var nodes = data.selectNodes("/*/ElementType");
  this.learnValues(xml, nodes);

  //investigate values
  var attrs = data.selectNodes("/*/AttributeType");
  this.learnValues(xml, attrs, "AttributeType");

  var parentcounters = data.selectNodes("//parent[@mincount and @maxcount]");
  this.learnCounters(xml, parentcounters);

  var xsd = xmllib.parse(data.transformNode(this.xslt));

//__xopus_alert(xsd.xml);
//test.value=schema;
//__xopus_alert(data.xml);

  return xsd;
}

function learnDistinct_schemabuilder(insertionPoint, nodes, holderName, counter) {
  var node, nodeName;
  for (var i=0; i<nodes.length; i++) {
    node = nodes.item(i);
    nodeName = node.nodeName;

    // don't process #document
    if (nodeName != "#document") {
      var thenode = insertionPoint.selectSingleNode(holderName+"[@name='"+nodeName+"']");
      if (!thenode) {
        thenode = xmllib.parse('<'+holderName+' name="'+nodeName+'" />').documentElement;
        insertionPoint.appendChild(thenode);
      }

      if (counter) {
        var cnt = new Number(xmllib.getAttribute(thenode, ".", "count", 0));
        cnt++;
        xmllib.setAttribute(thenode, ".", "count", cnt);
      }
    };
  }
}

function countOccurences_schemabuilder(xml, parentName, childName) {
  var list;
  if (parentName == "#document") list = xml.selectNodes("/"); else list = xml.selectNodes("//"+parentName);
  var parent, count;
  var result = new Array();
  var total = new Object();
  total.min=42000;
  total.max=0;

  for (var i=0; i<list.length; i++) {
    parent = list.item(i);
    var count = parent.selectNodes(childName).length;
    result[i]=count;
    total.min = Math.min(total.min, count);
    total.max = Math.max(total.max, count);
  }
  return total;
}

function learnParents_schemabuilder(insertionPoint, nodes, holderName) {
  var node, nodeName, parent;
  for (var i=0; i<nodes.length; i++) {
    node = nodes.item(i);
    nodeName = node.nodeName;
    if (holderName=="ElementType") {
      parent = node.parentNode;
    } else {
      parent=node.selectSingleNode("parent::*");
    }

    var place = insertionPoint.selectSingleNode(holderName+"[@name='"+nodeName+"']");

    // count number of times it occured in total for each parent
    this.learnDistinct(place, parent.selectNodes("."), "parent", true);


    // now, check minCount and maxCount: how many times does this nodetype occur min/max for this parenttype
    var parents = place.selectNodes("parent");
    var mincount=0;
    var maxcount=0;
    var count;
    var xpathname=nodeName;
    if (node.nodeType==2) xpathname="@"+nodeName;

    for (var j=0; j<parents.length; j++) {
      count = this.countOccurences(node, parent.nodeName, xpathname);
      xmllib.setAttribute(parents.item(j), ".", "mincount", count.min);
      xmllib.setAttribute(parents.item(j), ".", "maxcount", count.max);
    }
  }
}

function calculateProperties_schemabuilder(xml, nodeName, attrName) {
  if (!attrName) attrName=""; else attrName="/@"+attrName;
  var nodes = xml.selectNodes("//"+nodeName+attrName);
  var node, value;
  var properties = new Array();
  properties["integer"]=true;
  properties["text"]=false;
  properties["containsnodes"]=false;
  properties["isemptysomewhere"]=false;
  properties["html"]=false;
  for (var i=0; i<nodes.length; i++) {
    node = nodes.item(i);
    value="";
    switch (node.nodeType) {
      case 2:
        // attribute
        value = node.nodeValue;
        if (isNaN(value*1) || value=="") {
            properties["integer"]=false;
            properties["text"]=true;
        }

        break;
      default:
        // element
        value = node.text;
        var firstchild = node.firstChild;
        if (firstchild && (firstchild.nodeType == 3 || firstchild.nodeType == 4)) {
          properties["text"] = true;

          if (value.indexOf(">")>-1 || value.indexOf("<")>-1) {
            properties["html"] = true;
          }
        }
        break;
    }
    // adjust properties
    var int = new Number(value);
    if (isNaN(int) || value=="") properties["integer"] = false;

    var children = node.selectNodes("*");
    if (children.length>0) properties["containsnodes"]=true;

    if (value.length==0) properties["isemptysomewhere"]=true;
  }
  return properties;
}

function learnValues_schemabuilder(originalData, nodes, holderName) {
  var node, nodeName, properties;

  for (var i=0; i<nodes.length; i++) {
    node = nodes.item(i);
    nodeName = node.attributes.getNamedItem("name").nodeValue;
    if (holderName=="AttributeType") {
      properties = this.calculateProperties(originalData, "*", nodeName);
    } else {
      properties = this.calculateProperties(originalData, nodeName);
    }
    for (prop in properties) {
      xmllib.setAttribute(node, ".", prop, properties[prop]);
    }
  }
}

function learnCounters_schemabuilder(originalData, nodes) {
  var node, min, max, count;
  var minOccurs, maxOccurs;

  for (var i=0; i<nodes.length; i++) {
    node = nodes.item(i);
    min = node.attributes.getNamedItem("mincount").nodeValue*1;
    max = node.attributes.getNamedItem("maxcount").nodeValue*1;
    count = node.attributes.getNamedItem("count").nodeValue*1;

    minOccurs = 1;
    maxOccurs = -1;

    // decide what to do: what statistics decide minOccurs and maxOccurs?
    if (count<2) {
      // no examples. Cannot decide. Set defaults.
      if (min<1) minOccurs = 0;
      if (min>=1) minOccurs = 0;
      maxOccurs = -1;
    } else {
      // 2 or more examples. Let's say this we can decide some things
      if (min>=1) minOccurs = 1;
      if (min==0) minOccurs = 0;

      if (max==1) maxOccurs = 1;
      if (max==0 || max>1) maxOccurs = -1;
    }

    // store calculated values
    xmllib.setAttribute(node, ".", "minOccurs", minOccurs);
    if (maxOccurs==-1) maxOccurs="unbounded";
    xmllib.setAttribute(node, ".", "maxOccurs", maxOccurs);
  }
}


//constructor
function XMLLib() {
  var o = new Object();

  //properties
  o.msxml3 = true;
  try {
    xml = new ActiveXObject("MSXML2.DOMDocument.4.0");
    o.msxml3 = true;
    o.domid = "MSXML2.DOMDocument.4.0";
    o.freethreadeddomid = "MSXML2.FreeThreadedDOMDocument.4.0";
  } catch (e) {
    o.msxml3 = false;
    //**** auto install msxml3
    document.location.href="/xopus/msxml3/download.htm";
//status = "old MSXML DOM, which is OK";
    o.domid = "Microsoft.XMLDOM";
    o.freethreadeddomid = "Microsoft.FreeThreadedXMLDOM";
  }
//o.msxml3 = false;
//o.domid = "Microsoft.XMLDOM";
//o.freethreadeddomid = "Microsoft.FreeThreadedXMLDOM";

  //methods
  o.create = create_xmllib;
  o.load = load_xmllib;
  o.parse = parse_xmllib;
  o.process = process_xmllib;
  o.draw = draw_xmllib;
  o.getAttribute = getAttribute_xmllib;
  o.setAttribute = setAttribute_xmllib;
  o.send = send_xmllib;

  return o;
}
xmllib = new XMLLib();

function create_xmllib(freeThreaded) {
  var xml = null;
  if (freeThreaded && (freeThreaded == true)) xml = new ActiveXObject(this.freethreadeddomid);
  else xml = new ActiveXObject(this.domid);
  return xml;
}

function load_xmllib(xmlFile, freeThreaded) {
  ////////////////////////////////////////////////
  // loads an xml file and returns an XMLDOM object
  ////////////////////////////////////////////////

  try {
    var result = null;

    if (loadedxmls && (node = loadedxmls.selectSingleNode("/xml/file[(@href='" + xmlFile + "') or (@href2='" + xmlFile + "')]/*"))) {
      result = xmllib.parse(node.xml);
    } else {
      var xml = this.create(freeThreaded);
//      if (freeThreaded && (freeThreaded == true)) {
//        var xml = new ActiveXObject("Microsoft.FreeThreadedXMLDOM");
//      } else {
//        var xml = new ActiveXObject("Microsoft.XMLDOM");
//      }
      xml.async = false;
      xml.validateOnParse = true;
      if (!xml.load(xmlFile)) {
//        throw new Error(-42, "Error in /general/xml.js/xml_loadxml('" + xmlFile + "'): could not load the XML file called '" + xmlFile + "'");
result = null;
      } else {
        result = xml;
        xml = null;
      }
    }

    //replace $PATHROOT by the actual PATHROOT
    if (result) {
      var xml = result.xml;
      xml = xml.replace(/(\$PATHROOT)/g, PATHROOT);
      result = xmllib.parse(xml);
    }

    return result;
  } catch (e) {
//    if (e.number == -42) throw e;
//    else throw new Error(-42, "Error in /general/xml.js/xmllib.load('" + xmlFile + "'): " + e.description);
    return null;
  }
}

function parse_xmllib(xmltext, freeThreaded) {
  ////////////////////////////////////////////////
  // parses xml and returns an XMLDOM object
  ////////////////////////////////////////////////
  try {
    var xml = this.create(freeThreaded);
//    if (freeThreaded && (freeThreaded == true)) {
//      var xml = new ActiveXObject("Microsoft.FreeThreadedXMLDOM");
//    } else {
//      var xml = new ActiveXObject("Microsoft.XMLDOM");
//    }

//    var header = "";
//    if (xmltext.indexOf("<?xml version")==-1) header = '<?xml version="1.0" encoding="utf-8" ?>';
//    xmltext = header + xmltext;

    xml.loadXML(xmltext);
    if (!xml || !xml.documentElement) {
      throw new Error(-42, "Error in /general/xml.js/xmllib.parse('" + xmltext + "'): could not load the XML");
    }
    return xml;
  } catch (e) {
    if (e.number == -42) throw e;
    else throw new Error(-42, "Error in /general/xml.js/xmllib.parse('" + xmltext + "'): " + e.description);
  }
}

function process_xmllib(node, xsl) {
  try {
    //apply the xsl to the xml
    var theHTML = node.transformNode(xsl);

    //unescape the HTML
    theHTML = general.unescape(theHTML);

    //*****
    // Delete </br> since it will result in double breaks
    theHTML = theHTML.replace(/(><\/br>)/g, "/>");

    //return the HTML
    return theHTML;
  } catch (e) {
    if (e.number == -42) throw e;
    else throw new Error(-42, "Error in /general/xml.js/xmllib.process('" + node + "', '" + xsl + "'): " + e.description);
  }
}

function draw_xmllib(node, xsl, element) {
  try {
    //process the xml/xsl
    var theHTML = this.process(node, xsl);

    //show the new HTML
    element.innerHTML = theHTML;
  } catch (e) {
    if (e.number == -42) throw e;
    else throw new Error(-42, "Error in /general/xml.js/xmllib.draw('" + node + "', '" + xsl + "', '" + element + "'): " + e.description);
  }
}

function setAttribute_xmllib(node, pattern, name, value) {
  ////////////////////////////////////////////////
  // takes the 'node'
  // finds all elements matching 'pattern'
  // adds or updates the attribute named 'name' having value 'value'
  // if 'value' contains one or more # characters they will be replaced by the node-occurrence number
  ////////////////////////////////////////////////
  try {
    var dom = node;
    if (node.ownerDocument) dom = node.ownerDocument;

    var nodeList = node.selectNodes(pattern);
    for (var i=0; i<nodeList.length; i++) {
      var nodeAttr = nodeList[i].attributes;
      var newAttr = dom.createAttribute(name);
      value = "" + value;
      newAttr.nodeValue = value.replace(/(#)/g, i);
      nodeAttr.setNamedItem(newAttr);
    }
  } catch (e) {
    if (e.number == -42) throw e;
    else throw new Error(-42, "Error in /general/xml.js/xmllib.setAttribute('" + node + "', '" + pattern + "', '" + name + "', '" + value + "'): " + e.description);
  }
}

function getAttribute_xmllib(dom, pattern, name, defaultValue) {
  ////////////////////////////////////////////////
  // gets the value of the attribute called 'name' of the node specified by 'pattern' in the DOM called 'dom'
  // if the node or the attribute cannot be found the function will return null
  ////////////////////////////////////////////////
  try {
    var result = null;

    var node = dom.selectSingleNode(pattern + "/@" + name);
    if (node) result = node.nodeValue;

    //if no attribute found, return the default (if specified)
    if ((result==null) && defaultValue) result = defaultValue;

    return result;
  } catch (e) {
    if (e.number == -42) throw e;
    else throw new Error(-42, "Error in /general/xml.js/xmllib.getAttribute('" + dom + "', '" + pattern + "', '" + name + "', '" + value + "'): " + e.description);
  }
}

function send_xmllib(cmdURL, cmdXMLDOM, freeThreaded) {
  try {
    //create an XMLHTTP object
    var xmlHTTP = new ActiveXObject("Microsoft.XMLHTTP");

    //find a default for the command processor URL
    if (cmdURL == null) {
      __xopus_alert("no cmdurl");
      return;
    }

    //open a connection to the command processor
    xmlHTTP.Open("POST", cmdURL, false);

    //send the command
    xmlHTTP.Send(cmdXMLDOM);

    //receive the result and strip possible starting <?xml ...?> node
    var res = xmlHTTP.responseText;
    var pi = res.indexOf("?>");
    if (pi>0 && (pi < res.indexOf("\n"))) res = res.substring(pi+2);

    //create a response XML DOM object
    var resXML = this.parse(res, freeThreaded);

    if ((resXML == null) || (resXML.xml == "")) {
//__xopus_alert("er ging iets fout, de respons luidde: " + xmlHTTP.responseText);
      __xopus_alert("general/xml/xmllib.send('" + cmdURL + "', '" + cmdXMLDOM.xml + "') failed: could not load the response XML", null);
      return null;
    }
    return resXML;
  } catch (e) {
    if (e.number == -42) throw e;
    else throw new Error(-42, "Error in /general/xml.js/xmllib.send('" + cmdURL + "', '" + cmdXMLDOM.xml + "'): " + e.description);
  }
}



