////////////////////////////////////////////////////////////////////////////////////
//
// This Code Is Licensed Under The Xopus Open Source License
// More information: http://xopus.org, info@xopus.org
// Mailing list: http://groups.yahoo.com/group/xopus
//
// This product includes software developed by Q42 BV (http://www.q42.nl/).
// please visit http://www.q42.nl for more information or mail us: xopus@q42.nl
//
////////////////////////////////////////////////////////////////////////////////////

/**
 * Initialize the list of editor types. The order of the types is the priority order of the types.
 * The first type has the highest priority. The first type the accept the context will be the 
 * type used.
 *
 * At least one type must accept.
 */
EditorDriver.prototype.initializeEditorTypes = function(editorTools)
{
  this.editorTypes = [];
  this.editorTypes.push(new DefaultEditorType(editorTools));
}

/**
 * The default editor type.
 */
function DefaultEditorType(editorTools)
{
  this.editorTools = editorTools;
}

DefaultEditorType.prototype =
{
  accept: function(editor, context, storage)
  {
    return true;
  },
  
  create: function (editor, context, storage)
  {
    // Store the current situation
    storage.oldHTML = editor.innerHTML;
    
    if (storage.currentEditor)
      this.editorTools.showEditingBorder(editor);
    else if (storage.syncedEditor)
      this.editorTools.showSyncedBorder(editor);
    else
      this.editorTools.showDefaultBorder(editor);
  },
  
  destroy: function (editor, context, storage)
  {
  },  
  
  mouseOver: function (editor, context, storage)
  {
    if (!storage.currentEditor && !storage.syncedEditor)
       this.editorTools.showAttentionBorder(editor);
  },
  
  mouseOut: function (editor, context, storage)
  {
    if (!storage.currentEditor && !storage.syncedEditor)
      this.editorTools.hideAttentionBorder(editor);
  },
  
  start: function (editor, context, storage)
  {
    this.editorTools.showEditingBorder(editor);
    storage.currentEditor = true;
    
    // Store the current situation
    storage.oldHTML = editor.innerHTML;
  },
  
  stop: function (editor, context, storage)
  {
    this.editorTools.hideEditingBorder(editor);
    storage.currentEditor = false;
  },
  
  update: function (editor, context, newHTML, storage)
  {
    editor.innerHTML = newHTML;
    
    // Store the current situation
    storage.oldHTML = editor.innerHTML;
  },
  
  sync: function (editor, context, storage)
  {
    storage.syncedEditor = true;
    this.editorTools.showSyncedBorder(editor);
  },
  
  unsync: function (editor, context, storage)
  {
    this.editorTools.hideSyncedBorder(editor);
    storage.syncedEditor = false;
  },
  
  error: function (editor, context, storage)
  {
  },
  
  hasChanged: function (editor, context, storage)
  {
    return storage.oldHTML != editor.innerHTML;
  }
};

numScriptsLoading--;