/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.grammar.token.definition;

import java.io.Serializable;

import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.helpers.Decoder;

/**
 * This represents a set of character for a character class.<br/>
 * Like [abg]
 *
 * @author Stephan Michels
 * @version $Revision: 1.2 $ $Date: 2002/05/06 16:22:56 $
 */
public class CharacterSet implements CharacterClassElement, Serializable, Cloneable
{

  private String _set = "";

  public CharacterSet()
  {
  }

  public CharacterSet(CharacterSet cs)
  {
    _set = cs._set;
  }

  /**
   * Set the character set
   *
   * @param set Set of characters
   */
  public void setCharacters(String set)
  {
    _set = set;
  }

  /**
   * Returns the to comparing characters
   *
   * @return To comparing characters
   */
  public String getCharacters()
  {
    return _set;
  }

  /**
   * Convert the character set to a string
   *
   * @return Content of the alternation
   */
  public String toString()
  {
    return Decoder.decode(_set);
  }

  public Object clone() throws CloneNotSupportedException
  { 
    CharacterSet clone = new CharacterSet();

    clone.setCharacters(getCharacters());
    
    return clone;
  }
}
