/*
 *  Copyright (C) Chaperon. All rights reserved.
 *  -------------------------------------------------------------------------
 *  This software is published under the terms of the Apache Software License
 *  version 1.1, a copy of which has been included  with this distribution in
 *  the LICENSE file.
 */

package net.sourceforge.chaperon.parser.generator.conflict;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.grammar.symbol.Symbol;
import net.sourceforge.chaperon.parser.generator.ItemSet;

/**
 * This class presents a shift/reduce conflict
 *
 * @author Stephan Michels
 * @version %version%
 */
public class ShiftReduceConflict extends Conflict
{

  private ItemSet _itemset;

  private Symbol  _symbol;

  /**
   * Creates a shift/reduce conflict
   *
   * @param itemset Itemset, in which the conflict occurs
   * @param symbol Symbol
   */
  public ShiftReduceConflict(ItemSet itemset, Symbol symbol)
  {
    _itemset = itemset;
    _symbol = symbol;
  }

  /**
   * Returns the type of conflict
   *
   * @return Type of conflict
   */
  public String getType()
  {
    return "SHIFTREDUCE";
  }

  /**
   * Convert the conflict to a string
   *
   * @return Content of the conflict
   */
  public String toString()
  {
    return "Shift/Reduce conflict: state=" + _itemset + " symbol=" + _symbol;
  }
}
